#pragma once

#include <concepts>
#include <cstddef>
#include <ranges>

#include <memory>
#include <type_traits>

namespace gz::util {
    /// Satisfied when T is in PackTypes
    template<typename T, typename... PackTypes>
    concept IsInPack = (std::same_as<T, PackTypes> || ...); 

    template<typename T>
    concept False = false;

    /// Forward range holding integers
    template<typename T>
    concept IntegralForwardRange = std::ranges::forward_range<T> and std::integral<std::ranges::range_value_t<T>>;

    /// Forward range holding ValueType
	template<typename T, typename ValueType>
	concept ForwardRange = std::ranges::forward_range<T> and std::same_as<std::ranges::range_value_t<T>, ValueType>;

    /// Contiguous range holding ValueType
	template<typename T, typename ValueType>
	concept ContiguousRange = std::ranges::contiguous_range<T> and std::same_as<std::ranges::range_value_t<T>, ValueType>;

    /// ObjectType accessible via *t and convertible to bool
    template<typename T, typename ObjectType>
    concept IsPointerToType = requires (T t/*, std::ptrdiff_t idx*/) {
        /* { t.operator*() } -> std::same_as<std::add_lvalue_reference_t<ObjectType>>; */
        /* { t.operator->() } -> std::same_as<std::add_pointer_t<ObjectType>>; */
        /* { t.operator[](idx) } -> std::same_as<std::add_lvalue_reference_t<ObjectType>>; */
        { *t } -> std::same_as<std::add_lvalue_reference_t<ObjectType>>;
        /* { t-> } -> std::same_as<std::add_lvalue_reference_t<ObjectType>>; */
        { static_cast<bool>(t) };
    };
    static_assert(IsPointerToType<std::shared_ptr<int>, int>, "");
    static_assert(IsPointerToType<int*, int>, "");

 
}

/**
 * @file
 * @brief Concepts that might be useful elsewhere
 */
